==> refiners-0.2.0.tar.gz: refiners-0.2.0/src/refiners/foundationals/latent_diffusion/stable_diffusion_xl/t2i_adapter.py <==
from torch import Tensor

from refiners.foundationals.latent_diffusion.t2i_adapter import T2IAdapter, T2IFeatures, ConditionEncoderXL
from refiners.foundationals.latent_diffusion.stable_diffusion_xl import SDXLUNet
from refiners.foundationals.latent_diffusion.stable_diffusion_1.unet import ResidualAccumulator
import refiners.fluxion.layers as fl


class SDXLT2IAdapter(T2IAdapter[SDXLUNet]):
    def __init__(
        self,
        target: SDXLUNet,
        name: str,
        condition_encoder: ConditionEncoderXL | None = None,
        scale: float = 1.0,
        weights: dict[str, Tensor] | None = None,
    ) -> None:
        self.residual_indices = (3, 5, 8)  # the UNet's middle block is handled separately (see `inject` and `eject`)
        self._features = [T2IFeatures(name=name, index=i, scale=scale) for i in range(4)]
        super().__init__(
            target=target,
            name=name,
            condition_encoder=condition_encoder or ConditionEncoderXL(device=target.device, dtype=target.dtype),
            weights=weights,
        )

    def inject(self: "SDXLT2IAdapter", parent: fl.Chain | None = None) -> "SDXLT2IAdapter":
        def sanity_check_t2i(block: fl.Chain) -> None:
            for t2i_layer in block.layers(layer_type=T2IFeatures):
                assert t2i_layer.name != self.name, f"T2I-Adapter named {self.name} is already injected"

        # Note: `strict=False` because `residual_indices` is shorter than `_features` due to MiddleBlock (see below)
        for n, feat in zip(self.residual_indices, self._features, strict=False):
            block = self.target.DownBlocks[n]
            sanity_check_t2i(block)
            block.insert_before_type(ResidualAccumulator, feat)

        # Special case: the MiddleBlock has no ResidualAccumulator (this is done via a subsequent layer) so just append
        sanity_check_t2i(self.target.MiddleBlock)
        self.target.MiddleBlock.append(self._features[-1])
        return super().inject(parent)

    def eject(self: "SDXLT2IAdapter") -> None:
        # See `inject` re: `strict=False`
        for n, feat in zip(self.residual_indices, self._features, strict=False):
            self.target.DownBlocks[n].remove(feat)
        self.target.MiddleBlock.remove(self._features[-1])
        super().eject()

==> refiners-0.3.1.tar.gz: refiners-0.3.1/src/refiners/foundationals/latent_diffusion/stable_diffusion_xl/t2i_adapter.py <==
from torch import Tensor

import refiners.fluxion.layers as fl
from refiners.foundationals.latent_diffusion.stable_diffusion_1.unet import ResidualAccumulator
from refiners.foundationals.latent_diffusion.stable_diffusion_xl.unet import MiddleBlock, SDXLUNet
from refiners.foundationals.latent_diffusion.t2i_adapter import ConditionEncoderXL, T2IAdapter, T2IFeatures


class SDXLT2IAdapter(T2IAdapter[SDXLUNet]):
    def __init__(
        self,
        target: SDXLUNet,
        name: str,
        condition_encoder: ConditionEncoderXL | None = None,
        scale: float = 1.0,
        weights: dict[str, Tensor] | None = None,
    ) -> None:
        self.residual_indices = (3, 5, 8)  # the UNet's middle block is handled separately (see `inject` and `eject`)
        self._features = [T2IFeatures(name=name, index=i, scale=scale) for i in range(4)]
        super().__init__(
            target=target,
            name=name,
            condition_encoder=condition_encoder or ConditionEncoderXL(device=target.device, dtype=target.dtype),
            weights=weights,
        )

    def inject(self: "SDXLT2IAdapter", parent: fl.Chain | None = None) -> "SDXLT2IAdapter":
        def sanity_check_t2i(block: fl.Chain) -> None:
            for t2i_layer in block.layers(layer_type=T2IFeatures):
                assert t2i_layer.name != self.name, f"T2I-Adapter named {self.name} is already injected"

        # Note: `strict=False` because `residual_indices` is shorter than `_features` due to MiddleBlock (see below)
        for n, feat in zip(self.residual_indices, self._features, strict=False):
            block = self.target.layer(("DownBlocks", n), fl.Chain)
            sanity_check_t2i(block)
            block.insert_before_type(ResidualAccumulator, feat)

        # Special case: the MiddleBlock has no ResidualAccumulator (this is done via a subsequent layer) so just append
        mid_block = self.target.layer("MiddleBlock", MiddleBlock)
        sanity_check_t2i(mid_block)
        mid_block.append(self._features[-1])
        return super().inject(parent)

    def eject(self: "SDXLT2IAdapter") -> None:
        # See `inject` re: `strict=False`
        for n, feat in zip(self.residual_indices, self._features, strict=False):
            self.target.layer(("DownBlocks", n), fl.Chain).remove(feat)
        self.target.layer("MiddleBlock", MiddleBlock).remove(self._features[-1])
        super().eject()
